SystemOrganization addCategory: #'AntSimulation-Test'!
SystemOrganization addCategory: #'AntSimulation-Graphics'!
SystemOrganization addCategory: #AntSimulation!

BorderedMorph subclass: #GraphicalEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!GraphicalEnvironment methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 08:26'!
initialize
	super initialize.! !

GraphicalEnvironment subclass: #SurfaceEnvironmentView
	instanceVariableNames: 'cells env'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!SurfaceEnvironmentView class methodsFor: 'images' stamp: 'VanceThornton 7/16/2012 23:48'!
imageForAnt: direction
	|filename|

	filename := (AntSimulationManager imageDirectory), '/ant-', (direction symbol asString), '.png'.
	^ImageReadWriter formFromFileNamed: filename! !

!SurfaceEnvironmentView methodsFor: 'accessing' stamp: 'VanceThornton 7/17/2012 00:26'!
cellAt: row at: column
	"There should be a better way to do this, but the cells matrix is currently full of SurfaceEnvironmentViews."
	^(self submorphsSatisfying: [ :cell | (cell row == row) and: [cell column == column]]) at: 1! !

!SurfaceEnvironmentView methodsFor: 'accessing' stamp: 'agallan2 7/5/2012 08:38'!
cellsPerSide
	^ 30.! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'VanceThornton 7/11/2012 19:59'!
initialize
	| sampleCell width height n |
	super initialize.
	n := self cellsPerSide.
	sampleCell := AntSimulationCell new.
	width := sampleCell width.
	height := sampleCell height.
	self bounds: (5@5 extent: ((width*n) @(height*n)) + (2 * self borderWidth)).
	cells := Matrix new: n tabulate: [ :i :j | self newCellAt: i at: j ].
	self populate: 25.! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'VanceThornton 7/17/2012 00:39'!
newCellAt: row at: column
	"Creates a new AntSimulationCell at the given coordinates."
	| c origin |
	c := AntSimulationCell new.
	origin := self innerBounds origin.
	self addMorph: c.
	c row: row.
	c column: column.
	c position: ((column - 1) * c width) @ ((row - 1) * c height) + origin.
! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'VanceThornton 7/11/2012 21:29'!
populate: population
	"Create a random SurfaceEnvironment object and populate the graphical view appropriately."
	env := SurfaceEnvironment generateRandomWithRows: (self cellsPerSide) columns: (self cellsPerSide)! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'VanceThornton 7/17/2012 00:59'!
step
	"Create a random SurfaceEnvironment object and populate the graphical view appropriately."
	|cell cellView|

	"Clear previous drawing"
	(env entities) do: [ :entity |
		cell := env cellForEntity: entity.
		cellView := self cellAt: (cell row) at: (cell column).
		cellView removeAllMorphs.
		cellView color: (Color brown paler).
		cellView borderColor: (cellView color darker).
		cellView clearTarget.
		cellView entity: nil. "don't need this now but not necessarily a bad thing to think about"
	].

	env update.

	(env entities) do: [ :entity |
		|entityStatus|

		entityStatus := env statusForEntity: entity.
		
		cell := env cellForEntity: entity.
		cellView := self cellAt: (cell row) at: (cell column).

		(entity isKindOf: Ant) ifTrue: [
			cellView addImage: (SurfaceEnvironmentView imageForAnt: (entityStatus direction))
		].
	
		(entity isKindOf: Food) ifTrue: [
			cellView color: (Color green)
		].

	
		(entity isKindOf: Rock) ifTrue: [
			cellView color: (Color gray)
		].

		(entity isKindOf: Nest) ifTrue: [
			cellView color: (Color brown darker)
		].
			
		(entity isKindOf: Pheromone) ifTrue: [
			cellView color: (Color magenta lighter)
		].
	
		cellView borderColor: (cellView color darker).

				
		cellView entity: entity.
		cellView target: entity.
		cellView actionSelector: #report.
	].! !

SimpleButtonMorph subclass: #AntSimulationCell
	instanceVariableNames: 'row column entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!AntSimulationCell commentStamp: 'agallan2 7/4/2012 20:59' prior: 0!
I represent a single discrete location in the AntSimulation. I am a button, as clicking me returns information about the simulation object I represent.!

!AntSimulationCell methodsFor: 'accessing' stamp: 'VanceThornton 7/17/2012 00:04'!
addImage: image
	|imageMorph|
	imageMorph :=  ImageMorph new image: image.
	imageMorph position: self center - (imageMorph extent // 2).
	self addMorph: imageMorph.
	imageMorph lock
! !

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/7/2012 21:28'!
column
	^column! !

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/7/2012 21:28'!
column: n
	column := n! !

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/5/2012 08:25'!
defaultLabel
	^ ' '.! !

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/8/2012 16:10'!
entity
	^ entity.! !

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/8/2012 16:10'!
entity: aEntity
	entity := aEntity! !

!AntSimulationCell methodsFor: 'initialization' stamp: 'VanceThornton 7/17/2012 00:57'!
initialize
	super initialize.
	self borderWidth: 1.
	color := Color brown paler.
	self borderColor: (color darker).

	bounds := 0@0 corner: 24@24.
	entity := nil.! !

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/7/2012 21:28'!
row
	^row! !

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/7/2012 21:28'!
row: n
	row := n! !

PluggableSystemWindow subclass: #AntSimulationManager
	instanceVariableNames: 'environment'
	classVariableNames: 'imageDirectory'
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!AntSimulationManager class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 23:09'!
imageDirectory
	^imageDirectory! !

!AntSimulationManager class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 23:13'!
imageDirectory: newDirectory
	imageDirectory := newDirectory.! !

!AntSimulationManager class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/17/2012 01:02'!
initialize
	imageDirectory := 'ant-simulation'! !

!AntSimulationManager methodsFor: 'initialize-release' stamp: 'agallan2 7/15/2012 22:58'!
initialize
	super initialize.
	self setLabel: 'Ant Simulation'.
! !

!AntSimulationManager methodsFor: 'initialize-release' stamp: 'agallan2 7/16/2012 17:26'!
openInWorld
	environment := SurfaceEnvironmentView new.
	self addMorph: environment.
	"self initializeLabelArea." "seems better to use this but it doesn't work"
	self addLabelArea.
	self setFramesForLabelArea.
	self addCloseBox.
	self addCollapseBox.
	self addExpandBox.
	self addMenuControl. "would prefer this on the right"


	self openInWorldExtent: 500 @ 500. "this seems a bit of a hack"
! !

TestCase subclass: #EnvironmentTest
	instanceVariableNames: 'env entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!EnvironmentTest methodsFor: 'running' stamp: 'VanceThornton 7/7/2012 00:12'!
setUp
	|cell|
	env := Environment new initWithRows: 10 columns: 10.
	
	self assert: env rowCount equals: 10.
	self assert: env columnCount equals: 10.
	self assert: env entities size equals: 0.

	entity := Entity new.
	cell := env cellAtRow: 5 column: 5.
	env add: entity to: cell.

! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/10/2012 01:07'!
testAddEntity
	|newEntity cell entityStatus|

	newEntity := Entity new.	
	cell := env cellAtRow: 3 column: 2.
	env add: newEntity to: cell.
	
	"The environment contains one existing entity and the new entity"
	self assert: 2 equals: (env entities size).

	entityStatus := env statusForEntity: newEntity.
	self assert: 100 equals: (entityStatus energy).
	
	self assert: (entityStatus cell) == cell.
	self assert: (entityStatus cell) row equals: 3.
	self assert: (entityStatus cell) column equals: 2.! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/8/2012 10:41'!
testCellDirectionOfCell
	|cell otherCell|

	cell := env cellAtRow: 5 column: 7.
	
	otherCell := env cell: (Direction north) ofCell: cell.
	self assert: (otherCell) row equals: 4.
	self assert: (otherCell column) equals: 7.

	otherCell := env cell: (Direction northEast) ofCell: cell.
	self assert: (otherCell) row equals: 4.
	self assert: (otherCell column) equals: 8.

	otherCell := env cell: (Direction east) ofCell: cell.
	self assert: (otherCell) row equals: 5.
	self assert: (otherCell column) equals: 8.

	otherCell := env cell: (Direction southEast) ofCell: cell.
	self assert: (otherCell) row equals: 6.
	self assert: (otherCell column) equals: 8.
	
	otherCell := env cell: (Direction south) ofCell: cell.
	self assert: (otherCell) row equals: 6.
	self assert: (otherCell column) equals: 7.

	otherCell := env cell: (Direction southWest) ofCell: cell.
	self assert: (otherCell) row equals: 6.
	self assert: (otherCell column) equals: 6.

	otherCell := env cell: (Direction west) ofCell: cell.
	self assert: (otherCell) row equals: 5.
	self assert: (otherCell column) equals: 6.

	otherCell := env cell: (Direction northWest) ofCell: cell.
	self assert: (otherCell) row equals: 4.
	self assert: (otherCell column) equals: 6.

	"Test out of bounds cells"
	
	cell := env cellAtRow: 1 column: 7.
	self assert: (env cell: (Direction north) ofCell: cell) == nil.
	self assert: (env cell: (Direction northEast) ofCell: cell) == nil.
	self assert: (env cell: (Direction northWest) ofCell: cell) == nil.
	
	cell := env cellAtRow: (env rowCount) column: 7.
	self assert: (env cell: (Direction south) ofCell: cell) == nil.
	self assert: (env cell: (Direction southEast) ofCell: cell) == nil.
	self assert: (env cell: (Direction southWest) ofCell: cell) == nil.

	cell := env cellAtRow: 7 column: 1.
	self assert: (env cell: (Direction west) ofCell: cell) == nil.
	self assert: (env cell: (Direction northWest) ofCell: cell) == nil.
	self assert: (env cell: (Direction southWest) ofCell: cell) == nil.

	cell := env cellAtRow: 7 column: (env columnCount).
	self assert: (env cell: (Direction east) ofCell: cell) == nil.
	self assert: (env cell: (Direction northEast) ofCell: cell) == nil.
	self assert: (env cell: (Direction southEast) ofCell: cell) == nil.

	"Test invalid direction"
	self should: [env cell: #UP ofCell: cell] raise: Error.
	
	"Test nil cell"
	self should: [env cell: (Direction southEast) ofCell: nil] raise: Error.! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:11'!
testMoveEntity
	|toCell entityStatus|

	toCell := env cellAtRow: 3 column: 3.
	env move: entity to: toCell.
	
	self assert: env entities size equals: 1.

	entityStatus := env statusForEntity: entity.
	
	self assert: (entityStatus cell) == toCell.
	self assert: (entityStatus cell) row equals: 3.
	self assert: (entityStatus cell) column equals: 3.! !

EnvironmentTest subclass: #MoveMotorResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!MoveMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/12/2012 22:23'!
testMoveBackward
	|entityStatus cell response|

	entityStatus := env statusForEntity: entity.
	entityStatus direction: (Direction east).
	cell := entityStatus cell.
	
	response := MoveAction actionForEntity: entity direction: (MoveDirection backward).
	
	response performInEnvironment: env.
	
	self assert: (cell entities size) equals: 0.
	
	cell := entityStatus cell.
	self assert: (cell entities size) equals: 1.	
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: 4.

	"Test moving backward at the boundary of the environment"	
	cell := env cellAtRow: 5 column: 1.
	env move: entity to: cell.
	response performInEnvironment: env.

	"The entity should remain in the original cell"
	cell := entityStatus cell.
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: 1.! !

!MoveMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/12/2012 22:23'!
testMoveForward
	|entityStatus cell response|
	
	entityStatus := env statusForEntity: entity.
	entityStatus direction: (Direction east).
	cell := entityStatus cell.

	response := MoveAction actionForEntity: entity direction: (MoveDirection forward).
	
	response performInEnvironment: env.
	
	self assert: (cell entities size) equals: 0.
	
	cell := entityStatus cell.
	self assert: (cell entities size) equals: 1.	
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: 6.

	"Test moving forward at the boundary of the environment"	
	cell := env cellAtRow: 5 column: (env columnCount).
	env move: entity to: cell.
	response performInEnvironment: env.

	"The entity should remain in the original cell"
	cell := entityStatus cell.
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: (env columnCount).! !

EnvironmentTest subclass: #TurnMotorResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!TurnMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/12/2012 22:24'!
testTurnLeft
	|entityStatus cell response|

	entityStatus := env statusForEntity: entity.
	entityStatus direction: (Direction east).
	cell := entityStatus cell.
	
	response := TurnAction actionForEntity: entity direction: (TurnDirection left).	
	response performInEnvironment: env.

	self assert: entityStatus direction equals: (Direction northEast).
	self assert: entityStatus cell = cell.	
	
	"Turning left for the full circle (8 turns) should return the entity to the original direction"
	entityStatus direction: (Direction north).
	(1 to: 8) do: [:count|response performInEnvironment: env].
	self assert: entityStatus direction equals: (Direction north).
	
	
! !

!TurnMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/12/2012 22:24'!
testTurnRight
	|entityStatus cell response|

	entityStatus := env statusForEntity: entity.
	entityStatus direction: (Direction east).
	cell := entityStatus cell.
	
	response := TurnAction actionForEntity: entity direction: (TurnDirection right).
	response performInEnvironment: env.

	self assert: entityStatus direction equals: (Direction southEast).
	self assert: entityStatus cell = cell.	
	
	"Turning right for the full circle (8 turns) should return the entity to the original direction"
	entityStatus direction: (Direction north).
	(1 to: 8) do: [:count | response performInEnvironment: env].
	self assert: entityStatus direction equals: (Direction north).
	
	
! !

TestCase subclass: #SurfaceEnvironmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!SurfaceEnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/11/2012 21:50'!
testGenerateRandom
	|env|
	env := SurfaceEnvironment generateRandomWithRows: 25 columns: 25.
		
	(env entities) do: [:entity |
		|entityStatus cell|
		entityStatus := env statusForEntity: entity.
		self assert: entityStatus ~= nil.
		
		cell := entityStatus cell.
		self assert: ((cell entities) includes: entity).		
		self assert: (env cellAtRow: (cell row) column: (cell column)) == cell.
	].
! !

!SurfaceEnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/11/2012 21:51'!
testUpdate
	|env|
	env := SurfaceEnvironment generateRandomWithRows: 25 columns: 25.
	
	1000 timesRepeat: [
		env update
		"self shouldnt: [] raise: Error."		
	].! !

Object subclass: #Action
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Action methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 22:15'!
energyCost
	self subclassResponsibility! !

!Action methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:35'!
performInEnvironment: env
	self subclassResponsibility! !

Action subclass: #CompositeAction
	instanceVariableNames: 'responses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!CompositeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:55'!
add: response
	responses add: response! !

!CompositeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:59'!
energyCost
	|totalEnergyCost|
	
	totalEnergyCost := 0.
	
	responses do: [:response|
		totalEnergyCost := totalEnergyCost + (response energyCost)
	].

	^totalEnergyCost! !

!CompositeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:55'!
initialize
	responses := OrderedCollection new.! !

!CompositeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:57'!
performInEnvironment: env
	responses do: [:response | response performInEnvironment: env]
! !

!CompositeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:56'!
remove: response
	responses remove: response! !

Action subclass: #ConsumeAction
	instanceVariableNames: 'entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!ConsumeAction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:20'!
actionForEntity: entity
	^(ConsumeAction new entity: entity).! !

!ConsumeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:57'!
energyCost
	((entity class) respondsTo: #energyCostForConsumeResponse) ifTrue: [
		^(entity class) energyCostForConsumeResponse.
	].

	^Environment defaultEnergyCost! !

!ConsumeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 17:38'!
entity: anEntity
	entity := anEntity.! !

!ConsumeAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 23:06'!
performInEnvironment: env
	|entityStatus currentCell forwardCell otherEntity otherEntityStatus|

	entityStatus := env statusForEntity: entity.
	currentCell := entityStatus cell.
	forwardCell := env cell: (entityStatus direction) ofCell: currentCell.
		
	(forwardCell ~= nil and: [forwardCell entities size > 0]) ifTrue: [
		|energyTransferAmount|
		
		otherEntity := forwardCell entities atRandom: (AntSimulation random).
		otherEntityStatus := env statusForEntity: otherEntity.
		
		energyTransferAmount := (otherEntityStatus energy) min: (entityStatus maxEnergy).
		entityStatus increaseEnergy: energyTransferAmount.
		otherEntityStatus reduceEnergy: energyTransferAmount.
	].! !

Action subclass: #CreateEntityAction
	instanceVariableNames: 'entity newEntity energyCost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!CreateEntityAction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:20'!
actionForEntity: entity newEntity: newEntity
	^(CreateEntityAction new entity: entity) newEntity: newEntity.! !

!CreateEntityAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 21:59'!
energyCost
	^energyCost! !

!CreateEntityAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 21:38'!
entity: anEntity
	entity := anEntity! !

!CreateEntityAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 21:38'!
newEntity: anEntity
	newEntity := anEntity! !

!CreateEntityAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 22:17'!
performInEnvironment: env
	|entityStatus currentCell forwardCell|

	entityStatus := env statusForEntity: entity.
	currentCell := entityStatus cell.
	forwardCell := env cell: (Direction random) ofCell: currentCell.
		
	energyCost := 0.
	(forwardCell ~= nil and: [forwardCell entities size = 0]) ifTrue: [
		env add: newEntity to: forwardCell.
		energyCost := (env statusForEntity: newEntity) energy.
	].! !

Action subclass: #MoveAction
	instanceVariableNames: 'entity direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!MoveAction class methodsFor: 'class initialization' stamp: 'VanceThornton 7/12/2012 22:20'!
actionForEntity: entity direction: direction
	^(MoveAction new entity: entity) direction: direction.! !

!MoveAction methodsFor: 'initialize-release' stamp: 'VanceThornton 7/6/2012 23:38'!
direction: aDirection
	direction := aDirection.! !

!MoveAction methodsFor: 'action' stamp: 'VanceThornton 7/9/2012 23:29'!
energyCost
	((entity class) respondsTo: #energyCostForMoveResponse) ifTrue: [
		^(entity class) energyCostForMoveResponse.
	].

	^Environment defaultEnergyCost! !

!MoveAction methodsFor: 'initialize-release' stamp: 'VanceThornton 7/6/2012 22:56'!
entity: anEntity
	entity := anEntity.! !

!MoveAction methodsFor: 'action' stamp: 'VanceThornton 7/17/2012 00:40'!
performInEnvironment: env
	|entityStatus currentCell entityDirection newCell|

	entityStatus := env statusForEntity: entity.

	entityDirection := entityStatus direction.
	(direction == (MoveDirection backward)) ifTrue: [entityDirection := Direction reverseForDirection: entityDirection].
	
	currentCell := entityStatus cell.
	newCell := env cell: entityDirection ofCell: currentCell.
	
	newCell ~= nil ifTrue: [
		env move: entity to: newCell.
	].! !

MoveAction subclass: #MoveTowardAction
	instanceVariableNames: 'otherEntity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!MoveTowardAction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 18:52'!
actionForEntity: entity toward: otherEntity
	^(MoveTowardAction new entity: entity) otherEntity: otherEntity.! !

!MoveTowardAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 18:53'!
otherEntity: towardEntity
	otherEntity := towardEntity! !

!MoveTowardAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 19:32'!
performInEnvironment: env
	|entityStatus currentCell entityDirection otherEntityStatus targetCell newCell rowDelta columnDelta|

	entityStatus := env statusForEntity: entity.
	currentCell := entityStatus cell.

	otherEntityStatus := env statusForEntity: otherEntity.
	targetCell := otherEntityStatus cell.

	rowDelta := (currentCell row - targetCell row).
	columnDelta := (currentCell column - targetCell column).

	entityDirection :=  entityStatus direction.
	(rowDelta < 0) ifTrue: [entityDirection := Direction south].
	(rowDelta > 0) ifTrue: [entityDirection := Direction north].
	(columnDelta < 0) ifTrue: [entityDirection := Direction east].
	(columnDelta > 0) ifTrue: [entityDirection := Direction west].

	newCell := env cell: entityDirection ofCell: currentCell.
	
	newCell ~= nil ifTrue: [
		env move: entity to: newCell.
	].! !

Action subclass: #NoAction
	instanceVariableNames: 'entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!NoAction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:20'!
actionForEntity: entity
	^NoAction new entity: entity! !

!NoAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:29'!
energyCost
	((entity class) respondsTo: #energyCostForNoResponse) ifTrue: [
		^(entity class) energyCostForNoResponse.
	].

	^Environment defaultEnergyCost! !

!NoAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:19'!
entity: anEntity
	entity := anEntity! !

!NoAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:39'!
performInEnvironment: env! !

Action subclass: #PheromoneAction
	instanceVariableNames: 'entity type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!PheromoneAction class methodsFor: 'class initialization' stamp: 'VanceThornton 7/12/2012 22:21'!
actionForEntity: entity type: type
	^(PheromoneAction new entity: entity) type: type.! !

!PheromoneAction methodsFor: 'action' stamp: 'VanceThornton 7/10/2012 00:02'!
energyCost
	((entity class) respondsTo: #energyCostForPheromoneResponse) ifTrue: [
		^(entity class) energyCostForPheromoneResponse.
	].

	^Environment defaultEnergyCost! !

!PheromoneAction methodsFor: 'initialize-release' stamp: 'VanceThornton 7/10/2012 00:02'!
entity: anEntity
	entity := anEntity.! !

!PheromoneAction methodsFor: 'action' stamp: 'VanceThornton 7/10/2012 00:03'!
performInEnvironment: env
	|entityStatus|

	entityStatus := env statusForEntity: entity.
	env add: (Pheromone new) to: (entityStatus cell)
! !

!PheromoneAction methodsFor: 'initialize-release' stamp: 'VanceThornton 7/10/2012 00:04'!
type: aType
	type := aType! !

Action subclass: #TurnAction
	instanceVariableNames: 'entity direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!TurnAction class methodsFor: 'class initialization' stamp: 'VanceThornton 7/12/2012 22:21'!
actionForEntity: entity direction: direction
	^(TurnAction new entity: entity) direction: direction.! !

!TurnAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 18:31'!
direction: aDirection
	aDirection == nil ifTrue: [self error: 'Nil direction'].

	direction := aDirection.! !

!TurnAction methodsFor: 'action' stamp: 'VanceThornton 7/9/2012 23:30'!
energyCost
	((entity class) respondsTo: #energyCostForTurnResponse) ifTrue: [
		^(entity class) energyCostForTurnResponse.
	].

	^Environment defaultEnergyCost! !

!TurnAction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 23:35'!
entity: anEntity
	entity := anEntity.! !

!TurnAction methodsFor: 'action' stamp: 'VanceThornton 7/12/2012 22:16'!
performInEnvironment: env
	|entityStatus currentDirection newDirection|

	direction == nil ifTrue: [self error: 'Nil direction'].
	
	entityStatus := env statusForEntity: entity.
	currentDirection := entityStatus direction.

	newDirection := Direction directionAfterTurning: direction from: currentDirection.

	entityStatus direction: newDirection.
	
			
			
! !

Object subclass: #AntSimulation
	instanceVariableNames: 'env'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

AntSimulation class
	instanceVariableNames: 'random'!

AntSimulation class
	instanceVariableNames: 'random'!

!AntSimulation class methodsFor: 'class initialization' stamp: 'VanceThornton 7/8/2012 11:27'!
initialize
	Direction initialize.
	TurnDirection initialize.
	MoveDirection initialize.! !

!AntSimulation class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/4/2012 00:09'!
random
	(random == nil) ifTrue: [random := Random seed: 1].
	^random! !

Object subclass: #Cell
	instanceVariableNames: 'row column entities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Cell methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/3/2012 23:03'!
add: entity
	entities add: entity! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
column
	^column! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/4/2012 00:25'!
entities
	^entities! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/12/2012 22:28'!
entitiesOfKind: entityClass
	^entities select: [:entity | entity isKindOf: entityClass].! !

!Cell methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:24'!
initWithRow: newRow column: newColumn
 	row := newRow.
	column := newColumn.
	entities := Set new.! !

!Cell methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:49'!
remove: entity
	entities remove: entity! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
row
	^row! !

Object subclass: #Direction
	instanceVariableNames: 'symbol'
	classVariableNames: 'east leftTurnDictionary north northEast northWest reverseDirectionDictionary rightTurnDictionary south southEast southWest west'
	poolDictionaries: ''
	category: 'AntSimulation'!

!Direction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:15'!
directionAfterTurning: turnDirection from: currentDirection
	turnDirection == (TurnDirection left) ifTrue: [^Direction directionAfterTurningLeftFrom: currentDirection].
	turnDirection == (TurnDirection right) ifTrue: [^Direction directionAfterTurningRightFrom: currentDirection].
	self error: 'Invalid turn direction '.! !

!Direction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 10:13'!
directionAfterTurningLeftFrom: currentDirection
	^leftTurnDictionary at: currentDirection.! !

!Direction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 10:12'!
directionAfterTurningRightFrom: currentDirection
	^rightTurnDictionary at: currentDirection.! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:23'!
east
	^east! !

!Direction class methodsFor: 'class initialization' stamp: 'VanceThornton 7/16/2012 23:05'!
initialize
	north := Direction new.
	north symbol: #north.
	
	northEast := Direction new.
	northEast symbol: #northEast.
	
	east := Direction new.
	east symbol: #east.

	southEast := Direction new.
	southEast symbol: #southEast.

	south := Direction new.
	south symbol: #south.

	southWest := Direction new.
	southWest symbol: #southWest.

	west := Direction new.
	west symbol: #west.

	northWest := Direction new.
	northWest symbol: #northWest.
	
	leftTurnDictionary := Dictionary new.
	leftTurnDictionary add: (Association key: north value: northWest).
	leftTurnDictionary add: (Association key: northWest value: west).
	leftTurnDictionary add: (Association key: west value: southWest).
	leftTurnDictionary add: (Association key: southWest value: south).
	leftTurnDictionary add: (Association key: south value: southEast).
	leftTurnDictionary add: (Association key: southEast value: east).
	leftTurnDictionary add: (Association key: east value: northEast).
	leftTurnDictionary add: (Association key: northEast value: north).

	rightTurnDictionary := Dictionary new.
	rightTurnDictionary add: (Association key: north value: northEast).
	rightTurnDictionary add: (Association key: northEast value: east).
	rightTurnDictionary add: (Association key: east value: southEast).
	rightTurnDictionary add: (Association key: southEast value: south).
	rightTurnDictionary add: (Association key: south value: southWest).
	rightTurnDictionary add: (Association key: southWest value: west).
	rightTurnDictionary add: (Association key: west value: northWest).
	rightTurnDictionary add: (Association key: northWest value: north).
	
	reverseDirectionDictionary := Dictionary new.
	reverseDirectionDictionary add: (Association key: north value: south).
	reverseDirectionDictionary add: (Association key: northEast value: southWest).
	reverseDirectionDictionary add: (Association key: east value: west).
	reverseDirectionDictionary add: (Association key: southEast value: northWest).
	reverseDirectionDictionary add: (Association key: south value: north).
	reverseDirectionDictionary add: (Association key: southWest value: northEast).
	reverseDirectionDictionary add: (Association key: west value: east).
	reverseDirectionDictionary add: (Association key: northWest value: southEast).
	
! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:19'!
north
	^north! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:23'!
northEast
	^northEast! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:24'!
northWest
	^northWest! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/11/2012 22:15'!
random
	^(reverseDirectionDictionary keys) atRandom! !

!Direction class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 10:12'!
reverseForDirection: direction
	^reverseDirectionDictionary at: direction! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:24'!
south
	^south! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:23'!
southEast
	^southEast! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:24'!
southWest
	^southWest! !

!Direction class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:24'!
west
	^west! !

!Direction methodsFor: 'comparing' stamp: 'VanceThornton 7/8/2012 10:33'!
= aDirection
	^self == aDirection! !

!Direction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 23:03'!
symbol
	^symbol! !

!Direction methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 23:03'!
symbol: newSymbol
	symbol := newSymbol! !

Object subclass: #Entity
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Entity subclass: #ActiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

ActiveEntity subclass: #Ant
	instanceVariableNames: 'behavior nest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Ant class methodsFor: 'class initialization' stamp: 'VanceThornton 7/8/2012 00:07'!
withBehavior: behavior
	^Ant new behavior: behavior! !

!Ant methodsFor: 'initialize-release' stamp: 'VanceThornton 7/8/2012 00:06'!
behavior: newBehavior
	behavior := newBehavior! !

!Ant methodsFor: 'initialize-release' stamp: 'VanceThornton 7/16/2012 19:25'!
nest
	^nest! !

!Ant methodsFor: 'initialize-release' stamp: 'VanceThornton 7/16/2012 19:23'!
nest: theNest
	nest := theNest! !

!Ant methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 00:43'!
responseTo: sensoryInput
	^behavior responseTo: sensoryInput for: self! !

!Entity class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 00:03'!
energyLevelForPercentage: percentage
	(percentage > (2/3)) ifTrue: [^#HIGH].
	(percentage > (1/3)) ifTrue: [^#MEDIUM].
	^#LOW
	! !

!Entity class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 10:51'!
sensoryInputForEntity: entity inEnvironment: env
	|status direction onCell frontCell leftFrontCell rightFrontCell energyLevel|
	
	status := env statusForEntity: entity.
	
	direction := status direction.
	
	onCell := status cell.
	frontCell := env cell: (status direction) ofCell: onCell.
	leftFrontCell := env cell: (Direction directionAfterTurningLeftFrom: direction) ofCell: onCell.
	rightFrontCell := env cell: (Direction directionAfterTurningRightFrom: direction) ofCell: onCell.
	
	energyLevel := Entity energyLevelForPercentage: (status energyPercentage).
	
	^SensoryInput forEntityOnCell: onCell 
		frontCell: frontCell 
		leftFrontCell: leftFrontCell 
		rightFrontCell: rightFrontCell 
		energyLevel: energyLevel! !

!Entity methodsFor: 'comparing' stamp: 'VanceThornton 7/3/2012 18:58'!
= anEntity
	self == anEntity ifTrue: [^true].
	^(id = anEntity id)
! !

!Entity methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:58'!
id
	^id! !

!Entity methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:13'!
initialize
	id := UUID new.! !

!Entity methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/17/2012 00:17'!
report
	"Pop up a GUI window with basic info/status of self."
	UITheme builder message: 'I am an object of class ', self class asString, ' with unique identifier ', (self id) asString! !

!Entity methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:32'!
responseTo: sensoryInput
	self subclassResponsibility! !

Entity subclass: #InactiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

InactiveEntity subclass: #Food
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Food class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:33'!
energyCostForNoResponse
	^0.0! !

Food subclass: #Nest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Nest methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 21:49'!
minEnergy
	^250! !

!Nest methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 19:24'!
responseTo: sensoryInput
	
	(sensoryInput energyLevel = #HIGH or: [sensoryInput energyLevel = #MEDIUM]) ifTrue: [
		|newEntity|
		newEntity := Ant withBehavior: (ForagingAntBehavior new).
		newEntity nest: self.
		
		^CreateEntityAction actionForEntity: self newEntity: newEntity.
	].

	^NoAction new.! !

!InactiveEntity methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:23'!
responseTo: sensoryInput
	^NoAction actionForEntity: self.! !

InactiveEntity subclass: #Obstacle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Obstacle commentStamp: 'VanceThornton 7/12/2012 22:32' prior: 0!
Only one entity of type Obstacle can be in a cell at any given time!

Obstacle subclass: #Rock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Rock class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:32'!
energyCostForNoResponse
	^0.0! !

InactiveEntity subclass: #Pheromone
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Object subclass: #EntityBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

EntityBehavior subclass: #AntBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!AntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:53'!
cellHasFood: cell
	cell == nil ifTrue: [^false].
	^(cell entitiesOfKind: Food)	 size > 0.
! !

!AntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:54'!
cellHasPheromone: cell
	cell == nil ifTrue: [^false].
	^(cell entitiesOfKind: Pheromone) size > 0.
! !

!AntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:23'!
respondToFood: sensoryInput forEntity: entity
	(sensoryInput energyLevel = #HIGH) ifTrue: [
		^nil
	].

	(self cellHasFood: (sensoryInput frontCell)) ifTrue: [
		^ConsumeAction actionForEntity: entity
	].

	(self cellHasFood: (sensoryInput leftFrontCell)) ifTrue: [
		^TurnAction actionForEntity: entity direction: (TurnDirection left).
	].

	(self cellHasFood: (sensoryInput rightFrontCell)) ifTrue: [
		^TurnAction actionForEntity: entity direction: (TurnDirection right).
	].

	^nil! !

!AntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 23:33'!
respondToPheromone: sensoryInput forEntity: entity

	^nil.
		
	"
	Here is some code to avoid pheromones
	(sensoryInput frontCell) = nil ifTrue: [^nil].
	(sensoryInput leftFrontCell) = nil ifTrue: [^nil].
	(sensoryInput rightFrontCell) = nil ifTrue: [^nil].
	
	(self cellHasPheromone: (sensoryInput frontCell)) ifFalse: [^nil].

	(self cellHasPheromone: (sensoryInput leftFrontCell)) ifFalse: [
		(sensoryInput leftFrontCell entitiesOfKind: Obstacle) size = 0 ifTrue: [
			^TurnAction actionForEntity: entity direction: (TurnDirection left)
		]
	].

	(self cellHasPheromone: (sensoryInput rightFrontCell)) ifFalse: [
		(sensoryInput rightFrontCell entitiesOfKind: Obstacle) size = 0 ifTrue: [
			^TurnAction actionForEntity: entity direction: (TurnDirection right)	
		]
	].
	"! !

!AntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 18:25'!
responseTo: sensoryInput for: entity
	|action|
	
	action := self respondToFood: sensoryInput forEntity: entity.
	(action ~= nil) ifTrue: [^action].
	
	action := self respondToPheromone: sensoryInput forEntity: entity.
	(action ~= nil) ifTrue: [^action].
	
	^nil! !

AntBehavior subclass: #ForagingAntBehavior
	instanceVariableNames: 'randomMovementBehavior'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!ForagingAntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 18:24'!
initialize
	randomMovementBehavior := RandomMovementBehavior new.! !

!ForagingAntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 19:22'!
responseTo: sensoryInput for: entity
	|action|
	
	action := super responseTo: sensoryInput for: entity.
	(action ~= nil) ifTrue: [^action].
	
	"If the energy level is low attempt to return to the nest"
	(sensoryInput energyLevel = #LOW) ifTrue: [
		^MoveTowardAction actionForEntity: entity toward: (entity nest).
	].
	
	action := CompositeAction new.
	action add: (randomMovementBehavior responseTo: sensoryInput for: entity).
	^action
	! !

AntBehavior subclass: #RandomMovementAntBehavior
	instanceVariableNames: 'randomMovementBehavior'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!RandomMovementAntBehavior methodsFor: 'initialize-release' stamp: 'VanceThornton 7/8/2012 17:47'!
initialize
	randomMovementBehavior := RandomMovementBehavior new.! !

!RandomMovementAntBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/16/2012 18:30'!
responseTo: sensoryInput for: entity
	|action|
	
	action := super responseTo: sensoryInput for: entity.
	(action ~= nil) ifTrue: [^action].
	
	action := CompositeAction new.
	action add: (randomMovementBehavior responseTo: sensoryInput for: entity).
	action add: (PheromoneAction actionForEntity: entity type: #AVOID).
	^action
	! !

!EntityBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 00:42'!
responseTo: sensoryInput for: entity
	self subclassResponsibility! !

EntityBehavior subclass: #RandomMovementBehavior
	instanceVariableNames: 'turnDirection movesBeforeDirectionChange'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!RandomMovementBehavior methodsFor: 'initialize-release' stamp: 'VanceThornton 7/8/2012 18:32'!
initialize
	movesBeforeDirectionChange := 0.
	turnDirection := TurnDirection random.! !

!RandomMovementBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 16:52'!
maxMovesBeforeDirectionChange
	^7! !

!RandomMovementBehavior methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/12/2012 22:29'!
responseTo: sensoryInput for: entity
	|frontCell|

	frontCell := sensoryInput frontCell.

	((frontCell == nil) or: [(frontCell entitiesOfKind: Obstacle) size > 0]) ifTrue: [
		^TurnAction actionForEntity: entity direction: turnDirection	
	].

	(movesBeforeDirectionChange > 0) ifTrue: [	
		movesBeforeDirectionChange := movesBeforeDirectionChange - 1.
		^MoveAction actionForEntity: entity direction: (MoveDirection forward).
	].

	turnDirection := (TurnDirection random).
	movesBeforeDirectionChange := AntSimulation random nextInt: (self maxMovesBeforeDirectionChange).
	^TurnAction actionForEntity: entity direction: turnDirection	
! !

Object subclass: #EntityStatus
	instanceVariableNames: 'cell energy maxEnergy direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:36'!
cell
	^cell! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 00:50'!
cell: newCell
	cell := newCell! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 00:44'!
direction
	^direction! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/8/2012 18:24'!
direction: newDirection
	(direction == nil) ifTrue: [self error: 'Invalid direction'].
	direction := newDirection! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 22:18'!
energy
	^energy! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/8/2012 18:07'!
energy: newEnergy
	energy := newEnergy.! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 22:19'!
energyPercentage
	^energy / maxEnergy! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/9/2012 23:07'!
increaseEnergy: amount
	energy := (energy + amount) min: maxEnergy.
! !

!EntityStatus methodsFor: 'initialize-release' stamp: 'VanceThornton 7/10/2012 01:01'!
initWithCell: aCell
	cell := aCell.
	energy := 100.
	maxEnergy := 200.
	direction := Direction north.
	! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 22:18'!
maxEnergy
	^maxEnergy! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/11/2012 22:02'!
maxEnergy: newMaxEnergy
	maxEnergy := newMaxEnergy! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/9/2012 23:07'!
reduceEnergy: amount
	energy := (energy - amount) max: 0.! !

Object subclass: #Environment
	instanceVariableNames: 'cellMatrix entityStatusMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Environment class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/9/2012 23:40'!
defaultEnergyCost
	^1.0! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 22:04'!
add: entity to: cell
	|status|
	status := EntityStatus new initWithCell: cell.
	entityStatusMap add: (Association key: entity value: status).
	cell add: entity.
	^status! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/8/2012 18:21'!
"Responds with the cell in the given direction relative to the given cell"
cell: direction ofCell: cell
	cell == nil ifTrue: [self error: 'Nil cell'].
	direction == nil ifTrue: [self error: 'Nil direction'].
	
	direction = (Direction north) ifTrue: [^self cellAtRow: (cell row - 1) column: (cell column)].
	direction = (Direction northEast) ifTrue: [^self cellAtRow: (cell row - 1) column: (cell column + 1)].
	direction = (Direction east) ifTrue: [^self cellAtRow: (cell row) column: (cell column + 1)].
	direction = (Direction southEast) ifTrue: [^self cellAtRow: (cell row + 1) column: (cell column + 1)].
	direction = (Direction south) ifTrue: [^self cellAtRow: (cell row + 1) column: (cell column)].
	direction = (Direction southWest) ifTrue: [^self cellAtRow: (cell row + 1) column: (cell column - 1)].
	direction = (Direction west) ifTrue: [^self cellAtRow: (cell row) column: (cell column - 1)].
	direction = (Direction northWest) ifTrue: [^self cellAtRow: (cell row - 1) column: (cell column - 1)].
	
	self error: 'Invalid direction'.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/17/2012 00:32'!
"Returns the cell at the specified row and column.  If the row and/or column is out of range then nil is returned"
cellAtRow: row column: column
	row < 1 ifTrue: [^nil].
	row > (self rowCount) ifTrue: [^nil].
	column < 1 ifTrue: [^nil].
	column > (self columnCount) ifTrue: [^nil].
	
	^cellMatrix at: row at: column
	! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 00:41'!
cellForEntity: entity
	^(entityStatusMap at: entity) cell.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
columnCount
	^cellMatrix columnCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:15'!
entities
	^entityStatusMap keys! !

!Environment methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:06'!
initWithRows: rowCount columns: columnCount
	cellMatrix := Matrix rows: rowCount columns: columnCount.
	cellMatrix indicesDo: [:row :column| cellMatrix at: row at: column put: (Cell new initWithRow: row column: column)].
	entityStatusMap := Dictionary new.! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:48'!
move: entity to: toCell
	|entityStatus fromCell|
	entityStatus := self statusForEntity: entity.
	fromCell := entityStatus cell.
	
	fromCell remove: entity.
	toCell add: entity.
	entityStatus cell: toCell.
! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/11/2012 20:52'!
randomCell
	|row column|
	row := AntSimulation random nextInt: (self rowCount).
	column := AntSimulation random nextInt: (self columnCount).
	^self cellAtRow: row column: column.! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 18:04'!
remove: entity
	|status|
	status := self statusForEntity: entity.
	(status cell) remove: entity.
	entityStatusMap removeKey: entity.
	
! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
rowCount
	^cellMatrix rowCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:16'!
statusForEntity: entity
	^entityStatusMap at: entity! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/17/2012 00:28'!
update
	|entitiesToRemove|
	
	(self entities) do: [:entity |
		|entityStatus sensoryInput action energyCost|
	
		entityStatus := self statusForEntity: entity.
		sensoryInput := (entity class) sensoryInputForEntity: entity inEnvironment: self.	
		action := entity responseTo: sensoryInput.
		action performInEnvironment: self.	
		entityStatus reduceEnergy: (action energyCost).
	].

	"Remove entities with no energy"
	entitiesToRemove := OrderedCollection new.
	(self entities) do: [:entity |
		|entityStatus|
		entityStatus := self statusForEntity: entity.
		(entityStatus energy) <= 0 ifTrue: [entitiesToRemove add: entity].
	].
	
	entitiesToRemove do: [:entity | self remove: entity].
! !

Environment subclass: #SurfaceEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!SurfaceEnvironment class methodsFor: 'class initialization' stamp: 'VanceThornton 7/11/2012 22:07'!
generateRandomWithRows: rows columns: columns 
	
	|env rockCount nest nestStatus|

	env := SurfaceEnvironment new initWithRows: rows columns: columns.		

	nest := Nest new.
	nestStatus := env add: nest to: (env randomCell).
	nestStatus maxEnergy: 5000.
	nestStatus increaseEnergy: ((nestStatus maxEnergy) * (2/3)).
	
	rockCount := ((rows * columns) * 0.05) asInteger.
		
	rockCount timesRepeat: [
		|cell|
		cell := env randomCell.
		cell entities size = 0 ifTrue: [env add: (Rock new) to: cell].		
	].

	^env! !

!SurfaceEnvironment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/10/2012 00:46'!
addFood: quantity

! !

!SurfaceEnvironment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/10/2012 00:35'!
addFood: quantity nearRow: row column: column maxDistance: maxDistance
	quantity timesRepeat: [|foodRow foodColumn cell|
		foodRow := row + (AntSimulation random nextInt: maxDistance) - (maxDistance / 2).
		foodColumn := column + (AntSimulation random nextInt: maxDistance) - (maxDistance / 2).
		cell := self cellAtRow: foodRow column: foodColumn.
		
		((cell ~= nil) and: [cell entities size = 0]) ifTrue: [
			self add: (Food new) to: cell.
		]
	]! !

!SurfaceEnvironment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/10/2012 00:52'!
addFoodAtRandomLocation: quantity
	|row column distance|
	
	row := AntSimulation random nextInt: (self rowCount).
	column := AntSimulation random nextInt: (self columnCount).
	distance := 2 * (quantity raisedTo: 0.5).
	self addFood: quantity nearRow: row column: column maxDistance: distance.
! !

!SurfaceEnvironment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/10/2012 00:45'!
minFoodCount
	^5! !

!SurfaceEnvironment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/10/2012 01:06'!
update
	|foodCount|
	
	super update.
	
	foodCount := ((self entities) select: [:entity | (entity isKindOf: Food)]) size.
	
	(foodCount < (self minFoodCount)) ifTrue: [
		self addFoodAtRandomLocation: 25
	]! !

Object subclass: #MoveDirection
	instanceVariableNames: 'symbol'
	classVariableNames: 'backward forward'
	poolDictionaries: ''
	category: 'AntSimulation'!

!MoveDirection class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:57'!
backward
	^backward! !

!MoveDirection class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:57'!
forward
	^forward! !

!MoveDirection class methodsFor: 'class initialization' stamp: 'VanceThornton 7/8/2012 11:25'!
initialize
	forward := MoveDirection new.
	forward symbol: #forward.
	
	backward := MoveDirection new.
	backward symbol: #backward.! !

!MoveDirection methodsFor: 'comparing' stamp: 'VanceThornton 7/8/2012 10:59'!
= aDirection
	^self == aDirection! !

!MoveDirection methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 11:26'!
symbol: aSymbol
	symbol := aSymbol! !

Object subclass: #SensoryInput
	instanceVariableNames: 'onCell frontCell leftFrontCell rightFrontCell energyLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!SensoryInput class methodsFor: 'class initialization' stamp: 'VanceThornton 7/8/2012 00:00'!
forEntityOnCell: on frontCell: front leftFrontCell: leftFront rightFrontCell: rightFront energyLevel: energy
	^SensoryInput new forEntityOnCell: on frontCell: front leftFrontCell: leftFront rightFrontCell: rightFront energyLevel: energy! !

!SensoryInput methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 21:17'!
energyLevel
	^energyLevel! !

!SensoryInput methodsFor: 'initialize-release' stamp: 'VanceThornton 7/7/2012 23:59'!
forEntityOnCell: on frontCell: front leftFrontCell: leftFront rightFrontCell: rightFront energyLevel: energy
	onCell := on.
	frontCell := front.
	leftFrontCell := leftFront.
	rightFrontCell := rightFront.
	energyLevel := energy.! !

!SensoryInput methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 23:51'!
frontCell
	^frontCell! !

!SensoryInput methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 23:51'!
leftFrontCell
	^leftFrontCell! !

!SensoryInput methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 23:51'!
onCell
	^onCell! !

!SensoryInput methodsFor: 'accessing' stamp: 'VanceThornton 7/7/2012 23:52'!
rightFrontCell
	^rightFrontCell! !

Object subclass: #TurnDirection
	instanceVariableNames: 'symbol'
	classVariableNames: 'left right'
	poolDictionaries: ''
	category: 'AntSimulation'!

!TurnDirection class methodsFor: 'class initialization' stamp: 'VanceThornton 7/8/2012 11:24'!
initialize
	left := TurnDirection new.
	left symbol: #left.
	
	right := TurnDirection new.
	right symbol: #right.! !

!TurnDirection class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:58'!
left
	^left! !

!TurnDirection class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 16:43'!
random
	(AntSimulation random nextInt: 2) = 1 ifTrue: [^left].	
	^right! !

!TurnDirection class methodsFor: 'directions' stamp: 'VanceThornton 7/8/2012 10:58'!
right
	^right! !

!TurnDirection methodsFor: 'comparing' stamp: 'VanceThornton 7/8/2012 10:59'!
= aDirection
	^self == aDirection! !

!TurnDirection methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/8/2012 11:28'!
symbol: aSymbol
	symbol := aSymbol! !
TurnDirection initialize!
AntSimulation initialize!
AntSimulationManager initialize!
MoveDirection initialize!
Direction initialize!
